#!/usr/bin/python3
import unittest
import sshmanager
import sshmanager
import os
sshmanager.TMP_DIR = '/home/benhutc/obj/csvload/src/sextant/test_resources/tmp'


class TestSequence(unittest.TestCase):
    def setUp(self):
        self.manager = None

    def tearDown(self):
        if self.manager:
            self.manager.close()
            self.manager = None

    def get_manager(self, local_port=9643, remote_host='localhost', 
                    remote_port=9643, ssh_user=None):
        return sshmanager.SSHManager(local_port, remote_host, remote_port, ssh_user)

    def test_init(self):
        self.assertRaises(ValueError, self.get_manager, local_port='invalid port')
        self.assertRaises(ValueError, self.get_manager, remote_port='invalid port')

    def test_connect(self):
        # make a connection to localhost and ensure that tmp is created
        self.manager = self.get_manager()
        self.assertTrue(os.path.isdir(self.manager._tmp_dir))
        self.manager.close()
        self.assertFalse(os.path.isdir(self.manager._tmp_dir))
        self.manager = None

        # check connecion failure
        self.assertRaises(sshmanager.SSHConnectionError, self.get_manager, remote_host='invalid host')

    def test_files(self):
        genuine_file = 'test_resources/parser_test.c'
        genuine_file2 = 'test_resources/parser_test'
        absent_file = 'absent_file'

        self.manager = self.get_manager()
        # check sending no files fails
        self.assertRaises(ValueError, self.manager.send_to_tmp_dir, [])
        # and sending an non-existent file
        self.assertRaises(ValueError, self.manager.send_to_tmp_dir, [absent_file, genuine_file])
        
        self.manager.send_to_tmp_dir([genuine_file, genuine_file2])
        self.assertTrue(os.path.isfile(os.path.join(self.manager._tmp_dir, genuine_file.split('/')[-1])))
        self.assertTrue(os.path.isfile(os.path.join(self.manager._tmp_dir, genuine_file2.split('/')[-1])))

        self.manager.remove_from_tmp_dir([genuine_file, genuine_file2])
        self.assertFalse(os.path.isfile(os.path.join(self.manager._tmp_dir, 
                                               genuine_file.split('/')[-1])))
        self.assertFalse(os.path.isfile(os.path.join(self.manager._tmp_dir,
                                               genuine_file2.split('/')[-1])))


        self.manager.close()
        self.manager = None


if __name__ == '__main__':
    # no coverage for:
    #  specifying ssh user
    #  scp failure
    #  an error in closing the ssh connection
    #  another error in closing the ssh connection
    #  mkdir failure
    #  rmdir failure
    unittest.main()

